@@ -24,7 +24,12 @@
 --------------------------------------------------------------------
 
 {-# LANGUAGE CPP                   #-}
+#if __GLASGOW_HASKELL__ >= 702
 {-# LANGUAGE DefaultSignatures     #-}
+#endif
+{-# LANGUAGE DeriveFoldable        #-}
+{-# LANGUAGE DeriveFunctor         #-}
+{-# LANGUAGE DeriveTraversable     #-}
 {-# LANGUAGE FlexibleContexts      #-}
 {-# LANGUAGE FlexibleInstances     #-}
 {-# LANGUAGE MultiParamTypeClasses #-}
@@ -36,8 +41,12 @@
 {-# LANGUAGE Safe                  #-}
 #else
 {-# LANGUAGE OverlappingInstances  #-}
+#if __GLASGOW_HASKELL__ >= 704
 {-# LANGUAGE Trustworthy           #-}
 #endif
+#endif
+
+#define HASCBOOL MIN_VERSION_base(4,10,0)
 
 module Test.SmallCheck.Series (
   -- {{{
@@ -59,7 +68,7 @@ module Test.SmallCheck.Series (
   -- >instance Serial m a => Serial m (Tree a)
   --
   -- Here we enable the @DeriveGeneric@ extension which allows to derive 'Generic'
-  -- instance for our data type. Then we declare that @Tree a@ is an instance of
+  -- instance for our data type. Then we declare that @Tree@ @a@ is an instance of
   -- 'Serial', but do not provide any definitions. This causes GHC to use the
   -- default definitions that use the 'Generic' instance.
   --
@@ -88,7 +97,7 @@ module Test.SmallCheck.Series (
   -- >instance Serial m a => Serial m (Light a) where
   -- >  series = newtypeCons Light
   --
-  -- For data types with more than 4 fields define @consN@ as
+  -- For data types with more than 6 fields define @consN@ as
   --
   -- >consN f = decDepth $
   -- >  f <$> series
@@ -96,32 +105,32 @@ module Test.SmallCheck.Series (
   -- >    <~> series
   -- >    <~> ...    {- series repeated N times in total -}
 
-  -- ** What does consN do, exactly?
+  -- ** What does @consN@ do, exactly?
 
   -- | @consN@ has type
-  -- @(Serial t_1, ..., Serial t_N) => (t_1 -> ... -> t_N -> t) -> Series t@.
+  -- @(Serial t₁, ..., Serial tₙ) => (t₁ -> ... -> tₙ -> t) -> Series t@.
   --
-  -- @consN f@ is a series which, for a given depth @d > 0@, produces values of the
+  -- @consN@ @f@ is a series which, for a given depth \(d > 0\), produces values of the
   -- form
   --
-  -- >f x_1 ... x_N
+  -- >f x₁ ... xₙ
   --
-  -- where @x_i@ ranges over all values of type @t_i@ of depth up to @d-1@
-  -- (as defined by the 'series' functions for @t_i@).
+  -- where @xₖ@ ranges over all values of type @tₖ@ of depth up to \(d-1\)
+  -- (as defined by the 'series' functions for @tₖ@).
   --
-  -- @consN@ functions also ensure that x_i are enumerated in the
+  -- @consN@ functions also ensure that xₖ are enumerated in the
   -- breadth-first order. Thus, combinations of smaller depth come first
-  -- (assuming the same is true for @t_i@).
+  -- (assuming the same is true for @tₖ@).
   --
-  -- If @d <= 0@, no values are produced.
+  -- If \(d \le 0\), no values are produced.
 
-  cons0, cons1, cons2, cons3, cons4, newtypeCons,
+  cons0, cons1, cons2, cons3, cons4, cons5, cons6, newtypeCons,
   -- * Function Generators
 
   -- | To generate functions of an application-specific argument type,
   -- make the type an instance of 'CoSerial'.
   --
-  -- Again there is a standard pattern, this time using the altsN
+  -- Again there is a standard pattern, this time using the @altsN@
   -- combinators where again N is constructor arity.  Here are @Tree@ and
   -- @Light@ instances:
   --
@@ -142,7 +151,7 @@ module Test.SmallCheck.Series (
   -- >      case l of
   -- >        Light x -> f x
   --
-  -- For data types with more than 4 fields define @altsN@ as
+  -- For data types with more than 6 fields define @altsN@ as
   --
   -- >altsN rs = do
   -- >  rs <- fixDepth rs
@@ -154,32 +163,34 @@ module Test.SmallCheck.Series (
   -- ** What does altsN do, exactly?
 
   -- | @altsN@ has type
-  -- @(Serial t_1, ..., Serial t_N) => Series t -> Series (t_1 -> ... -> t_N -> t)@.
+  -- @(Serial t₁, ..., Serial tₙ) => Series t -> Series (t₁ -> ... -> tₙ -> t)@.
   --
-  -- @altsN s@ is a series which, for a given depth @d@, produces functions of
+  -- @altsN@ @s@ is a series which, for a given depth \( d \), produces functions of
   -- type
   --
-  -- >t_1 -> ... -> t_N -> t
+  -- >t₁ -> ... -> tₙ -> t
   --
-  -- If @d <= 0@, these are constant functions, one for each value produced
+  -- If \( d \le 0 \), these are constant functions, one for each value produced
   -- by @s@.
   --
-  -- If @d > 0@, these functions inspect each of their arguments up to the depth
-  -- @d-1@ (as defined by the 'coseries' functions for the corresponding
+  -- If \( d > 0 \), these functions inspect each of their arguments up to the depth
+  -- \( d-1 \) (as defined by the 'coseries' functions for the corresponding
   -- types) and return values produced by @s@. The depth to which the
   -- values are enumerated does not depend on the depth of inspection.
 
-  alts0, alts1, alts2, alts3, alts4, newtypeAlts,
+  alts0, alts1, alts2, alts3, alts4, alts5, alts6, newtypeAlts,
 
   -- * Basic definitions
   Depth, Series, Serial(..), CoSerial(..),
 
+#if __GLASGOW_HASKELL__ >= 702
   -- * Generic implementations
   genericSeries,
   genericCoseries,
+#endif
 
   -- * Convenient wrappers
-  Positive(..), NonNegative(..), NonEmpty(..),
+  Positive(..), NonNegative(..), NonZero(..), NonEmpty(..),
 
   -- * Other useful definitions
   (\/), (><), (<~>), (>>-),
@@ -200,15 +211,30 @@ module Test.SmallCheck.Series (
 import Control.Monad (liftM, guard, mzero, mplus, msum)
 import Control.Monad.Logic (MonadLogic, (>>-), interleave, msplit, observeAllT)
 import Control.Monad.Reader (ask, local)
-import Control.Applicative (empty, pure, (<$>))
+import Control.Applicative (empty, pure, (<$>), (<|>))
+import Data.Complex (Complex(..))
+import Data.Foldable (Foldable)
+import Data.Functor.Compose (Compose(..))
+import Data.Void (Void, absurd)
 import Control.Monad.Identity (Identity(..))
 import Data.Int (Int, Int8, Int16, Int32, Int64)
 import Data.List (intercalate)
+import qualified Data.List.NonEmpty as NE
 import Data.Ratio (Ratio, numerator, denominator, (%))
+import Data.Traversable (Traversable)
 import Data.Word (Word, Word8, Word16, Word32, Word64)
+import Foreign.C.Types (CFloat(..), CDouble(..), CChar(..), CSChar(..), CUChar(..), CShort(..), CUShort(..), CInt(..), CUInt(..), CLong(..), CULong(..), CPtrdiff(..), CSize(..), CWchar(..), CSigAtomic(..), CLLong(..), CULLong(..), CIntPtr(..), CUIntPtr(..), CIntMax(..), CUIntMax(..), CClock(..), CTime(..))
+#if __GLASGOW_HASKELL__ >= 702
+import Foreign.C.Types (CUSeconds(..), CSUSeconds(..))
+#endif
+#if HASCBOOL
+import Foreign.C.Types (CBool(..))
+#endif
 import Numeric.Natural (Natural)
 import Test.SmallCheck.SeriesMonad
-import GHC.Generics (Generic, (:+:)(..), (:*:)(..), C1, K1(..), M1(..), U1(..), Rep, to, from)
+#if __GLASGOW_HASKELL__ >= 702
+import GHC.Generics (Generic, (:+:)(..), (:*:)(..), C1, K1(..), M1(..), U1(..), V1(..), Rep, to, from)
+#endif
 
 ------------------------------
 -- Main types and classes
@@ -218,13 +244,17 @@ import GHC.Generics (Generic, (:+:)(..), (:*:)(..), C1, K1(..), M1(..), U1(..),
 class Monad m => Serial m a where
   series   :: Series m a
 
+#if __GLASGOW_HASKELL__ >= 704
   default series :: (Generic a, GSerial m (Rep a)) => Series m a
   series = genericSeries
+#endif
 
+#if __GLASGOW_HASKELL__ >= 702
 genericSeries
   :: (Monad m, Generic a, GSerial m (Rep a))
   => Series m a
 genericSeries = to <$> gSeries
+#endif
 
 class Monad m => CoSerial m a where
   -- | A proper 'coseries' implementation should pass the depth unchanged to
@@ -232,13 +262,18 @@ class Monad m => CoSerial m a where
   -- functions non-uniform in their arguments.
   coseries :: Series m b -> Series m (a->b)
 
+#if __GLASGOW_HASKELL__ >= 704
   default coseries :: (Generic a, GCoSerial m (Rep a)) => Series m b -> Series m (a->b)
   coseries = genericCoseries
+#endif
 
+#if __GLASGOW_HASKELL__ >= 702
 genericCoseries
   :: (Monad m, Generic a, GCoSerial m (Rep a))
   => Series m b -> Series m (a->b)
 genericCoseries rs = (. from) <$> gCoseries rs
+#endif
+
 -- }}}
 
 ------------------------------
@@ -253,24 +288,23 @@ generate f = do
   d <- getDepth
   msum $ map return $ f d
 
--- | Limit a 'Series' to its first @n@ elements
+-- | Limit a 'Series' to its first @n@ elements.
 limit :: forall m a . Monad m => Int -> Series m a -> Series m a
 limit n0 (Series s) = Series $ go n0 s
   where
-    go :: MonadLogic ml => Int -> ml b -> ml b
-    go 0 _ = mzero
+    go 0 _ = empty
     go n mb1 = do
       cons :: Maybe (b, ml b) <- msplit mb1
       case cons of
-        Nothing -> mzero
-        Just (b, mb2) -> return b `mplus` go (n-1) mb2
+        Nothing -> empty
+        Just (b, mb2) -> return b <|> go (n-1) mb2
 
 suchThat :: Series m a -> (a -> Bool) -> Series m a
 suchThat s p = s >>= \x -> if p x then pure x else empty
 
--- | Given a depth, return the list of values generated by a Serial instance.
+-- | Given a depth, return the list of values generated by a 'Serial' instance.
 --
--- Example, list all integers up to depth 1:
+-- For example, list all integers up to depth 1:
 --
 -- * @listSeries 1 :: [Int]   -- returns [0,1,-1]@
 listSeries :: Serial Identity a => Depth -> [a]
@@ -281,21 +315,20 @@ listSeries d = list d series
 --
 -- Examples:
 --
--- * @list 3 'series' :: [Int]                  -- returns [0,1,-1,2,-2,3,-3]@
+-- * @'list' 3 'series' :: ['Int']                  -- returns [0,1,-1,2,-2,3,-3]@
 --
--- * @list 3 ('series' :: 'Series' 'Identity' Int)  -- returns [0,1,-1,2,-2,3,-3]@
+-- * @'list' 3 ('series' :: 'Series' 'Data.Functor.Identity' 'Int')  -- returns [0,1,-1,2,-2,3,-3]@
 --
--- * @list 2 'series' :: [[Bool]]               -- returns [[],[True],[False]]@
+-- * @'list' 2 'series' :: [['Bool']]               -- returns [[],['True'],['False']]@
 --
 -- The first two are equivalent. The second has a more explicit type binding.
 list :: Depth -> Series Identity a -> [a]
 list d s = runIdentity $ observeAllT $ runSeries d s
 
--- | Monadic version of 'list'
-listM :: Monad m => Depth -> Series m a -> m [a]
+-- | Monadic version of 'list'.
 listM d s = observeAllT $ runSeries d s
 
--- | Sum (union) of series
+-- | Sum (union) of series.
 infixr 7 \/
 (\/) :: Monad m => Series m a -> Series m a -> Series m a
 (\/) = interleave
@@ -305,7 +338,7 @@ infixr 8 ><
 (><) :: Monad m => Series m a -> Series m b -> Series m (a,b)
 a >< b = (,) <$> a <~> b
 
--- | Fair version of 'ap' and '<*>'
+-- | Fair version of 'Control.Applicative.ap' and '<*>'.
 infixl 4 <~>
 (<~>) :: Monad m => Series m (a -> b) -> Series m a -> Series m b
 a <~> b = a >>- (<$> b)
@@ -316,17 +349,23 @@ uncurry3 f (x,y,z) = f x y z
 uncurry4 :: (a->b->c->d->e) -> ((a,b,c,d)->e)
 uncurry4 f (w,x,y,z) = f w x y z
 
--- | Query the current depth
+uncurry5 :: (a->b->c->d->e->f) -> ((a,b,c,d,e)->f)
+uncurry5 f (v,w,x,y,z) = f v w x y z
+
+uncurry6 :: (a->b->c->d->e->f->g) -> ((a,b,c,d,e,f)->g)
+uncurry6 f (u,v,w,x,y,z) = f u v w x y z
+
+-- | Query the current depth.
 getDepth :: Series m Depth
 getDepth = Series ask
 
--- | Run a series with a modified depth
+-- | Run a series with a modified depth.
 localDepth :: (Depth -> Depth) -> Series m a -> Series m a
 localDepth f (Series a) = Series $ local f a
 
 -- | Run a 'Series' with the depth decreased by 1.
 --
--- If the current depth is less or equal to 0, the result is 'mzero'.
+-- If the current depth is less or equal to 0, the result is 'empty'.
 decDepth :: Series m a -> Series m a
 decDepth a = do
   checkDepth
@@ -395,6 +434,25 @@ cons4 f = decDepth $
     <~> series
     <~> series
 
+cons5 :: (Serial m a, Serial m b, Serial m c, Serial m d, Serial m e) =>
+         (a->b->c->d->e->f) -> Series m f
+cons5 f = decDepth $
+  f <$> series
+    <~> series
+    <~> series
+    <~> series
+    <~> series
+
+cons6 :: (Serial m a, Serial m b, Serial m c, Serial m d, Serial m e, Serial m f) =>
+         (a->b->c->d->e->f->g) -> Series m g
+cons6 f = decDepth $
+  f <$> series
+    <~> series
+    <~> series
+    <~> series
+    <~> series
+    <~> series
+
 alts0 :: Series m a -> Series m a
 alts0 s = s
 
@@ -428,6 +486,22 @@ alts4 rs = do
     (constM $ constM $ constM $ constM rs)
     (coseries $ coseries $ coseries $ coseries rs)
 
+alts5 ::  (CoSerial m a, CoSerial m b, CoSerial m c, CoSerial m d, CoSerial m e) =>
+            Series m f -> Series m (a->b->c->d->e->f)
+alts5 rs = do
+  rs <- fixDepth rs
+  decDepthChecked
+    (constM $ constM $ constM $ constM $ constM rs)
+    (coseries $ coseries $ coseries $ coseries $ coseries rs)
+
+alts6 ::  (CoSerial m a, CoSerial m b, CoSerial m c, CoSerial m d, CoSerial m e, CoSerial m f) =>
+            Series m g -> Series m (a->b->c->d->e->f->g)
+alts6 rs = do
+  rs <- fixDepth rs
+  decDepthChecked
+    (constM $ constM $ constM $ constM $ constM $ constM rs)
+    (coseries $ coseries $ coseries $ coseries $ coseries $ coseries rs)
+
 -- | Same as 'alts1', but preserves the depth.
 newtypeAlts :: CoSerial m a => Series m b -> Series m (a->b)
 newtypeAlts = coseries
@@ -444,6 +518,7 @@ class GSerial m f where
 class GCoSerial m f where
   gCoseries :: Series m b -> Series m (f a -> b)
 
+#if __GLASGOW_HASKELL__ >= 702
 instance {-# OVERLAPPABLE #-} GSerial m f => GSerial m (M1 i c f) where
   gSeries = M1 <$> gSeries
   {-# INLINE gSeries #-}
@@ -465,6 +540,13 @@ instance GCoSerial m U1 where
   gCoseries rs = constM rs
   {-# INLINE gCoseries #-}
 
+instance GSerial m V1 where
+  gSeries = mzero
+  {-# INLINE gSeries #-}
+instance GCoSerial m V1 where
+  gCoseries = const $ return (\a -> a `seq` let x = x in x)
+  {-# INLINE gCoseries #-}
+
 instance (Monad m, GSerial m a, GSerial m b) => GSerial m (a :*: b) where
   gSeries = (:*:) <$> gSeries <~> gSeries
   {-# INLINE gSeries #-}
@@ -490,6 +572,8 @@ instance (Monad m, GCoSerial m a, GCoSerial m b) => GCoSerial m (a :+: b) where
 instance {-# OVERLAPPING #-} GSerial m f => GSerial m (C1 c f) where
   gSeries = M1 <$> decDepth gSeries
   {-# INLINE gSeries #-}
+#endif
+
 -- }}}
 
 ------------------------------
@@ -529,7 +613,7 @@ instance Monad m => CoSerial m Word64 where coseries = fmap (. N) . coseries
 -- | 'N' is a wrapper for 'Integral' types that causes only non-negative values
 -- to be generated. Generated functions of type @N a -> b@ do not distinguish
 -- different negative values of @a@.
-newtype N a = N { unN :: a } deriving (Eq, Ord)
+newtype N a = N { unN :: a } deriving (Eq, Ord, Show)
 
 instance Real a => Real (N a) where
   toRational (N x) = toRational x
@@ -570,7 +654,7 @@ instance (Integral a, Monad m) => CoSerial m (N a) where
         else z
 
 -- | 'M' is a helper type to generate values of a signed type of increasing magnitude.
-newtype M a = M { unM :: a } deriving (Eq, Ord)
+newtype M a = M { unM :: a } deriving (Eq, Ord, Show)
 
 instance Real a => Real (M a) where
   toRational (M x) = toRational x
@@ -655,6 +739,16 @@ instance (Serial m a, Serial m b, Serial m c, Serial m d) => Serial m (a,b,c,d)
 instance (CoSerial m a, CoSerial m b, CoSerial m c, CoSerial m d) => CoSerial m (a,b,c,d) where
   coseries rs = uncurry4 <$> alts4 rs
 
+instance (Serial m a, Serial m b, Serial m c, Serial m d, Serial m e) => Serial m (a,b,c,d,e) where
+  series = cons5 (,,,,)
+instance (CoSerial m a, CoSerial m b, CoSerial m c, CoSerial m d, CoSerial m e) => CoSerial m (a,b,c,d,e) where
+  coseries rs = uncurry5 <$> alts5 rs
+
+instance (Serial m a, Serial m b, Serial m c, Serial m d, Serial m e, Serial m f) => Serial m (a,b,c,d,e,f) where
+  series = cons6 (,,,,,)
+instance (CoSerial m a, CoSerial m b, CoSerial m c, CoSerial m d, CoSerial m e, CoSerial m f) => CoSerial m (a,b,c,d,e,f) where
+  coseries rs = uncurry6 <$> alts6 rs
+
 instance Monad m => Serial m Bool where
   series = cons0 True \/ cons0 False
 instance Monad m => CoSerial m Bool where
@@ -663,6 +757,18 @@ instance Monad m => CoSerial m Bool where
     rs >>- \r2 ->
     return $ \x -> if x then r1 else r2
 
+instance Monad m => Serial m Ordering where
+  series = cons0 LT \/ cons0 EQ \/ cons0 GT
+instance Monad m => CoSerial m Ordering where
+  coseries rs =
+    rs >>- \r1 ->
+    rs >>- \r2 ->
+    rs >>- \r3 ->
+    pure $ \x -> case x of
+        LT -> r1
+        EQ -> r2
+        GT -> r3
+
 instance (Serial m a) => Serial m (Maybe a) where
   series = cons0 Nothing \/ cons1 Just
 instance (CoSerial m a) => CoSerial m (Maybe a) where
@@ -683,6 +789,36 @@ instance CoSerial m a => CoSerial m [a] where
     alts2 rs >>- \f ->
     return $ \xs -> case xs of [] -> y; x:xs' -> f x xs'
 
+instance Serial m a => Serial m (NE.NonEmpty a) where
+  series = cons2 (NE.:|)
+
+instance CoSerial m a => CoSerial m (NE.NonEmpty a) where
+  coseries rs =
+    alts2 rs >>- \f ->
+    return $ \(x NE.:| xs') -> f x xs'
+
+#if MIN_VERSION_base(4,4,0)
+instance Serial m a => Serial m (Complex a) where
+#else
+instance (RealFloat a, Serial m a) => Serial m (Complex a) where
+#endif
+  series = cons2 (:+)
+
+#if MIN_VERSION_base(4,4,0)
+instance CoSerial m a => CoSerial m (Complex a) where
+#else
+instance (RealFloat a, CoSerial m a) => CoSerial m (Complex a) where
+#endif
+  coseries rs =
+    alts2 rs >>- \f ->
+    return $ \(x :+ xs') -> f x xs'
+
+instance Monad m => Serial m Void where
+  series = mzero
+
+instance Monad m => CoSerial m Void where
+  coseries = const $ return absurd
+
 instance (CoSerial m a, Serial m b) => Serial m (a->b) where
   series = coseries series
 -- Thanks to Ralf Hinze for the definition of coseries
@@ -727,6 +863,11 @@ instance (Serial Identity a, Show a, Show b) => Show (a -> b) where
     height = length . lines
     (widthLimit,lengthLimit,depthLimit) = (80,20,3)::(Int,Int,Depth)
 
+instance (Monad m, Serial m (f (g a))) => Serial m (Compose f g a) where
+  series = Compose <$> series
+instance (Monad m, CoSerial m (f (g a))) => CoSerial m (Compose f g a) where
+  coseries = fmap (. getCompose) . coseries
+
 -- }}}
 
 ------------------------------
@@ -735,13 +876,17 @@ instance (Serial Identity a, Show a, Show b) => Show (a -> b) where
 -- {{{
 
 --------------------------------------------------------------------------
--- | @Positive x@: guarantees that @x \> 0@.
+-- | 'Positive' @x@ guarantees that \( x > 0 \).
 newtype Positive a = Positive { getPositive :: a }
- deriving (Eq, Ord)
+ deriving (Eq, Ord, Functor, Foldable, Traversable)
 
 instance Real a => Real (Positive a) where
   toRational (Positive x) = toRational x
 
+instance (Num a, Bounded a) => Bounded (Positive a) where
+  minBound = Positive 1
+  maxBound = Positive (maxBound :: a)
+
 instance Enum a => Enum (Positive a) where
   toEnum x = Positive (toEnum x)
   fromEnum (Positive x) = fromEnum x
@@ -766,13 +911,17 @@ instance (Num a, Ord a, Serial m a) => Serial m (Positive a) where
 instance Show a => Show (Positive a) where
   showsPrec n (Positive x) = showsPrec n x
 
--- | @NonNegative x@: guarantees that @x \>= 0@.
+-- | 'NonNegative' @x@ guarantees that \( x \ge 0 \).
 newtype NonNegative a = NonNegative { getNonNegative :: a }
- deriving (Eq, Ord)
+ deriving (Eq, Ord, Functor, Foldable, Traversable)
 
 instance Real a => Real (NonNegative a) where
   toRational (NonNegative x) = toRational x
 
+instance (Num a, Bounded a) => Bounded (NonNegative a) where
+  minBound = NonNegative 0
+  maxBound = NonNegative (maxBound :: a)
+
 instance Enum a => Enum (NonNegative a) where
   toEnum x = NonNegative (toEnum x)
   fromEnum (NonNegative x) = fromEnum x
@@ -797,7 +946,42 @@ instance (Num a, Ord a, Serial m a) => Serial m (NonNegative a) where
 instance Show a => Show (NonNegative a) where
   showsPrec n (NonNegative x) = showsPrec n x
 
--- | @NonEmpty xs@: guarantees that @xs@ is not null
+-- | 'NonZero' @x@ guarantees that \( x \ne 0 \).
+newtype NonZero a = NonZero { getNonZero :: a }
+ deriving (Eq, Ord, Functor, Foldable, Traversable)
+
+instance Real a => Real (NonZero a) where
+  toRational (NonZero x) = toRational x
+
+instance (Eq a, Num a, Bounded a) => Bounded (NonZero a) where
+  minBound = let x = minBound in NonZero (if x == 0 then  1 else x)
+  maxBound = let x = maxBound in NonZero (if x == 0 then -1 else x)
+
+instance Enum a => Enum (NonZero a) where
+  toEnum x = NonZero (toEnum x)
+  fromEnum (NonZero x) = fromEnum x
+
+instance Num a => Num (NonZero a) where
+  NonZero x + NonZero y = NonZero (x + y)
+  NonZero x * NonZero y = NonZero (x * y)
+  negate (NonZero x) = NonZero (negate x)
+  abs (NonZero x) = NonZero (abs x)
+  signum (NonZero x) = NonZero (signum x)
+  fromInteger x = NonZero (fromInteger x)
+
+instance Integral a => Integral (NonZero a) where
+  quotRem (NonZero x) (NonZero y) = (NonZero q, NonZero r)
+    where
+      (q, r) = x `quotRem` y
+  toInteger (NonZero x) = toInteger x
+
+instance (Num a, Ord a, Serial m a) => Serial m (NonZero a) where
+  series = NonZero <$> series `suchThat` (/= 0)
+
+instance Show a => Show (NonZero a) where
+  showsPrec n (NonZero x) = showsPrec n x
+
+-- | 'NonEmpty' @xs@ guarantees that @xs@ is not null.
 newtype NonEmpty a = NonEmpty { getNonEmpty :: [a] }
 
 instance (Serial m a) => Serial m (NonEmpty a) where
@@ -807,3 +991,144 @@ instance Show a => Show (NonEmpty a) where
   showsPrec n (NonEmpty x) = showsPrec n x
 
 -- }}}
+
+------------------------------
+-- Foreign.C.Types
+------------------------------
+-- {{{
+
+#if MIN_VERSION_base(4,5,0)
+instance Monad m => Serial m CFloat where
+  series = newtypeCons CFloat
+instance Monad m => CoSerial m CFloat where
+  coseries rs = newtypeAlts rs >>- \f -> return $ \l -> case l of CFloat x -> f x
+
+instance Monad m => Serial m CDouble where
+  series = newtypeCons CDouble
+instance Monad m => CoSerial m CDouble where
+  coseries rs = newtypeAlts rs >>- \f -> return $ \l -> case l of CDouble x -> f x
+
+#if HASCBOOL
+instance Monad m => Serial m CBool where
+  series = newtypeCons CBool
+instance Monad m => CoSerial m CBool where
+  coseries rs = newtypeAlts rs >>- \f -> return $ \l -> case l of CBool x -> f x
+#endif
+
+instance Monad m => Serial m CChar where
+  series = newtypeCons CChar
+instance Monad m => CoSerial m CChar where
+  coseries rs = newtypeAlts rs >>- \f -> return $ \l -> case l of CChar x -> f x
+
+instance Monad m => Serial m CSChar where
+  series = newtypeCons CSChar
+instance Monad m => CoSerial m CSChar where
+  coseries rs = newtypeAlts rs >>- \f -> return $ \l -> case l of CSChar x -> f x
+
+instance Monad m => Serial m CUChar where
+  series = newtypeCons CUChar
+instance Monad m => CoSerial m CUChar where
+  coseries rs = newtypeAlts rs >>- \f -> return $ \l -> case l of CUChar x -> f x
+
+instance Monad m => Serial m CShort where
+  series = newtypeCons CShort
+instance Monad m => CoSerial m CShort where
+  coseries rs = newtypeAlts rs >>- \f -> return $ \l -> case l of CShort x -> f x
+
+instance Monad m => Serial m CUShort where
+  series = newtypeCons CUShort
+instance Monad m => CoSerial m CUShort where
+  coseries rs = newtypeAlts rs >>- \f -> return $ \l -> case l of CUShort x -> f x
+
+instance Monad m => Serial m CInt where
+  series = newtypeCons CInt
+instance Monad m => CoSerial m CInt where
+  coseries rs = newtypeAlts rs >>- \f -> return $ \l -> case l of CInt x -> f x
+
+instance Monad m => Serial m CUInt where
+  series = newtypeCons CUInt
+instance Monad m => CoSerial m CUInt where
+  coseries rs = newtypeAlts rs >>- \f -> return $ \l -> case l of CUInt x -> f x
+
+instance Monad m => Serial m CLong where
+  series = newtypeCons CLong
+instance Monad m => CoSerial m CLong where
+  coseries rs = newtypeAlts rs >>- \f -> return $ \l -> case l of CLong x -> f x
+
+instance Monad m => Serial m CULong where
+  series = newtypeCons CULong
+instance Monad m => CoSerial m CULong where
+  coseries rs = newtypeAlts rs >>- \f -> return $ \l -> case l of CULong x -> f x
+
+instance Monad m => Serial m CPtrdiff where
+  series = newtypeCons CPtrdiff
+instance Monad m => CoSerial m CPtrdiff where
+  coseries rs = newtypeAlts rs >>- \f -> return $ \l -> case l of CPtrdiff x -> f x
+
+instance Monad m => Serial m CSize where
+  series = newtypeCons CSize
+instance Monad m => CoSerial m CSize where
+  coseries rs = newtypeAlts rs >>- \f -> return $ \l -> case l of CSize x -> f x
+
+instance Monad m => Serial m CWchar where
+  series = newtypeCons CWchar
+instance Monad m => CoSerial m CWchar where
+  coseries rs = newtypeAlts rs >>- \f -> return $ \l -> case l of CWchar x -> f x
+
+instance Monad m => Serial m CSigAtomic where
+  series = newtypeCons CSigAtomic
+instance Monad m => CoSerial m CSigAtomic where
+  coseries rs = newtypeAlts rs >>- \f -> return $ \l -> case l of CSigAtomic x -> f x
+
+instance Monad m => Serial m CLLong where
+  series = newtypeCons CLLong
+instance Monad m => CoSerial m CLLong where
+  coseries rs = newtypeAlts rs >>- \f -> return $ \l -> case l of CLLong x -> f x
+
+instance Monad m => Serial m CULLong where
+  series = newtypeCons CULLong
+instance Monad m => CoSerial m CULLong where
+  coseries rs = newtypeAlts rs >>- \f -> return $ \l -> case l of CULLong x -> f x
+
+instance Monad m => Serial m CIntPtr where
+  series = newtypeCons CIntPtr
+instance Monad m => CoSerial m CIntPtr where
+  coseries rs = newtypeAlts rs >>- \f -> return $ \l -> case l of CIntPtr x -> f x
+
+instance Monad m => Serial m CUIntPtr where
+  series = newtypeCons CUIntPtr
+instance Monad m => CoSerial m CUIntPtr where
+  coseries rs = newtypeAlts rs >>- \f -> return $ \l -> case l of CUIntPtr x -> f x
+
+instance Monad m => Serial m CIntMax where
+  series = newtypeCons CIntMax
+instance Monad m => CoSerial m CIntMax where
+  coseries rs = newtypeAlts rs >>- \f -> return $ \l -> case l of CIntMax x -> f x
+
+instance Monad m => Serial m CUIntMax where
+  series = newtypeCons CUIntMax
+instance Monad m => CoSerial m CUIntMax where
+  coseries rs = newtypeAlts rs >>- \f -> return $ \l -> case l of CUIntMax x -> f x
+
+instance Monad m => Serial m CClock where
+  series = newtypeCons CClock
+instance Monad m => CoSerial m CClock where
+  coseries rs = newtypeAlts rs >>- \f -> return $ \l -> case l of CClock x -> f x
+
+instance Monad m => Serial m CTime where
+  series = newtypeCons CTime
+instance Monad m => CoSerial m CTime where
+  coseries rs = newtypeAlts rs >>- \f -> return $ \l -> case l of CTime x -> f x
+
+instance Monad m => Serial m CUSeconds where
+  series = newtypeCons CUSeconds
+instance Monad m => CoSerial m CUSeconds where
+  coseries rs = newtypeAlts rs >>- \f -> return $ \l -> case l of CUSeconds x -> f x
+
+instance Monad m => Serial m CSUSeconds where
+  series = newtypeCons CSUSeconds
+instance Monad m => CoSerial m CSUSeconds where
+  coseries rs = newtypeAlts rs >>- \f -> return $ \l -> case l of CSUSeconds x -> f x
+#endif
+
+-- }}}
